/*
 * tmos - bdi32.c
 * Copyright (C) 2010  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * Tiny RT BDI32 modules support
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <tmos/kernel.h>
#include <tmos/trt.h>
#include <tmos/adc.h>
#include <tmos/jiffies.h>

#define ADDR_0	P2_3
#define ADDR_1	P2_4
#define ADDR_2	P2_5
#define ADDR_3	P2_7
#define BYTE_HI	P2_2
#define module_ready()	(!(P2_6))

static char bdi32_read(unsigned char addr, uint16_t __data *x) __reentrant
{
	uint8_t lo, hi;

	ADDR_0 = (addr >> 0) & 1;
	ADDR_1 = (addr >> 1) & 1;
	ADDR_2 = (addr >> 2) & 1;
	ADDR_3 = (addr >> 3) & 1;
	BYTE_HI = 0;
	lo = 0;
	hi = 0;

	if (module_ready()) {
		lo = ~P0;
	} else {
		goto err;
	}

	BYTE_HI = 1;
	if (module_ready()) {
		hi = ~P0;
	} else {
		goto err;
	}
	*x = (hi << 8) | lo;
	return 0;
err:
	return 1;
}

void trt_bdi32_read(void) __reentrant
{
	taddr_t i;
	uint16_t status;

	status = 0xffff;
	for (i = 0; i < CONFIG_TRT_BDI32_COUNT; i++) {
		uint16_t v = 0;
		char ret;
		
		ret = bdi32_read(i, &v);
		if (!ret)
			status &= ~(1 << i);
		trt_write_reg(v, CONFIG_TRT_BDI32_INPUTBASE + 1 + i);
	}
	trt_write_reg(status, CONFIG_TRT_BDI32_INPUTBASE);
}
