/*
 * tmos - context.h
 *
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __ASM_TASK_H
#define __ASM_TASK_H

#include <tmos/kernel.h>
#include <setjmp.h>
#include <asm/iram.h>
#include <tmos/jiffies.h>

typedef unsigned char pid_t;

#if CONFIG_MULTITASKING
/*
 * thread context - each thread have this context structure.
 */
struct context {
	unsigned char sp;		/* stack pointer */
#ifdef CONFIG_TASK_TIMES
	jiffies_t cpu_time;		/* used cpu time */
#endif
	
#ifdef CONFIG_TDATA
	/*
	 * in TDATA memory model each process have paged iram area.
	 * tdata_page specifies xdata page number.
	 *
	 * For threads without TDATA memory tdata_page should be 0
	 * (but this is currently not supported).
	 */
	unsigned char tdata_page;
#endif

	/*
	 * pointer to context that should preempt this context, for
	 * simple round-robin scheduling contexts can be linked
	 * circular list.
	 */
	struct context __kernel *next;
};
#else
struct context {
	char dummy;
};
#endif

#if CONFIG_MULTITASKING
/* FIXME: this is probably broken */
#define GET_WCHAN(x)	(be16_to_cpu(*((uint16_t __data *) ((x)->sp - 4))))
#else
#define GET_WCHAN(x)	(0)
#endif

#if CONFIG_USER_PREEMPT
#define __thread_switch_attr __using 2
#else
#define __thread_switch_attr
#endif

#define stack_alloc(size) ((unsigned char) iram_alloc((size)))

#if CONFIG_USE_IDLE
extern __bit __need_wakeup;

#define need_wakeup()		(__need_wakeup)
#define set_need_wakeup()	do { __need_wakeup = 1; } while (0)
void idle(void);

#else
#define need_wakeup()		(0)
#define set_need_wakeup()	do { } while (0)
#define idle()			do { } while (0)
#endif

#ifdef CONFIG_TDATA

#define TASK_DATA_BEGIN(name) struct __task_data_##name {
#define TASK_DATA_END(name) };

#define TASK_DATA(name, varname) \
	(((struct __task_data_##name __data *) TDATA_BASE)->varname)

#define TASK_STACK() (TDATA_BASE)
#define TASK_STACK_SIZE(size) (TDATA_BASE)
#define TASK_STACK_P(name) (TDATA_BASE + sizeof(struct __task_data_##name))
#define TASK_STACK_PSIZE(name, size) TASK_STACK(name)

#else

#define TASK_DATA_BEGIN(name) struct {
#define TASK_DATA_END(name) } __task_data_##name;

#define TASK_DATA(name, varname) \
	(__task_data_##name.varname)

#define TASK_STACK() (stack_alloc(64))
#define TASK_STACK_SIZE(size) (stack_alloc((size)))
#define TASK_STACK_P(name) (stack_alloc(64))
#define TASK_STACK_PSIZE(name, size) (stack_alloc((size)))

#endif



#if CONFIG_MULTITASKING

#if CONFIG_TDATA

#if CONFIG_ADUC841
#include <asm/aduc841.h>

#define WANT_PUSH_DP		1

#define switch_mm_copy_byte() 						\
	do {								\
		__asm							\
			movx	A, @DPTR				\
		__endasm;						\
		__asm							\
			xch	A, @r0					\
		__endasm;						\
		__asm							\
			movx	@DPTR, A				\
		__endasm;						\
		__asm							\
			inc	r0					\
		__endasm;						\
	} while (0)

#define switch_mm()							\
	do {								\
		register unsigned char old_page;			\
		register unsigned char new_page;			\
									\
		/* switch current and remember data pages */		\
		old_page = current->tdata_page;				\
		current = current->next;				\
		new_page = current->tdata_page;				\
									\
		DPH = old_page;						\
		DPL = 0;						\
									\
		DPCON = DPCON_AUTO | DPCON_SP_INC | DPCON_MP_INC	\
				| DPCON_DPSEL;				\
		DPH = new_page;						\
									\
		DPL = 0x80;						\
		__asm							\
			mov	r0, DPL					\
		__endasm;						\
		DPL = 0;						\
		__asm							\
01000$:									\
		__endasm;						\
		switch_mm_copy_byte();					\
		switch_mm_copy_byte();					\
		switch_mm_copy_byte();					\
		switch_mm_copy_byte();					\
									\
		switch_mm_copy_byte();					\
		switch_mm_copy_byte();					\
		switch_mm_copy_byte();					\
		switch_mm_copy_byte();					\
									\
		__asm							\
			mov	a, r0					\
		__endasm;						\
		__asm							\
			jnz	01000$					\
		__endasm;						\
		DPCON = 0x00;						\
	} while (0)
#else
#define AUTO_PUSH_DP		1
#endif

#define STORE_MEM() \
	do { \
		register unsigned char __data *s \
			= (unsigned char __data *) 0x80; \
		register unsigned char __xdata *d \
			= (unsigned char __xdata *) \
			(current->tdata_page << 8); \
		for (; s;) \
			*d++ = *s++; \
	} while (0)

#define LOAD_MEM() \
	do { \
		register unsigned char __data *d \
			= (unsigned char __data *) 0x80; \
		register unsigned char __xdata *s \
			= (unsigned char __xdata *) \
			(current->tdata_page << 8); \
		for (; d;) \
			*d++ = *s++; \
	} while (0)

#else

#if CONFIG_USER_PREEMPT
#define WANT_PUSH_DP		1
#endif

#if AUTO_PUSH_DP && WANT_PUSH_DP
#warning "DP is saved twice"
#endif

#define STORE_MEM()	do { } while (0)
#define LOAD_MEM()	do { } while (0)
#endif

#if WANT_PUSH_DP
#define PUSH_DP() \
	do { \
		__asm \
			push dpl; \
		__endasm; \
		__asm \
			push dph; \
		__endasm; \
	} while (0)
#define POP_DP() \
	do { \
		__asm \
			pop dph \
			pop dpl \
		__endasm; \
	} while (0)
#else
#define PUSH_DP()		do { } while (0)
#define POP_DP()		do { } while (0)
#endif

#define PUSH_REG(x) \
	do { \
		__asm \
			__TEMPREG = x; \
		__endasm; \
		__asm \
			push __TEMPREG \
		__endasm; \
	} while (0)

#define POP_REG(x) \
	do { \
		__asm \
			__TEMPREG = x; \
		__endasm; \
		__asm \
			pop __TEMPREG \
		__endasm; \
	} while (0)

#if CONFIG_USER_PREEMPT
#define PUSH_REGS() \
	do { \
		PUSH_REG(0x00); \
		PUSH_REG(0x01); \
		PUSH_REG(0x02); \
		PUSH_REG(0x03); \
		PUSH_REG(0x04); \
		PUSH_REG(0x05); \
		PUSH_REG(0x06); \
		PUSH_REG(0x07); \
		__asm \
			push b;  \
		__endasm; \
	} while (0)

#define POP_REGS() \
	do { \
		__asm \
			pop b; \
		__endasm; \
		POP_REG(0x07); \
		POP_REG(0x06); \
		POP_REG(0x05); \
		POP_REG(0x04); \
		POP_REG(0x03); \
		POP_REG(0x02); \
		POP_REG(0x01); \
		POP_REG(0x00); \
	} while (0)
#else
#define PUSH_REGS()	do { } while (0)
#define POP_REGS()	do { } while (0)
#endif


#if CONFIG_USER_PREEMPT && !CONFIG_KERNEL_PREEMPT
#define irq_preempt_enabled() \
	preempt_enabled_flags(*((unsigned char __data *) SP))
#else
#define irq_preempt_enabled() preempt_enabled()
#endif

#ifndef switch_mm
#define switch_mm()							\
	do {								\
		STORE_MEM();						\
		current = current->next;				\
		LOAD_MEM();						\
	} while (0)
#endif

/*
 * TASK_SWITCH - do context switch
 *
 * This macro should be used in interrupt, thread_switch() should
 * be used in other contexts. In most cases this macro should be called
 * via irq_switch_task().
 *
 * Make sure than stack is correct, before TASK_SWITCH() is should have:
 *
 * 	RETURN_ADDR_0
 * 	RETURN_ADDR_1
 * 	PSW
 * 	[DPL]	- only in TDATA model on !ADUC841
 * 	[DPH]	- only in TDATA model on !ADUC841
 * 	ACC
 *
 * Also preemption should be possible:
 * 	- at least user preemption should be enabled
 * 	- current task should be in user mode or in preemptive kernel
 * 	  section
 */
#define TASK_SWITCH() \
	do { \
		PUSH_DP(); \
		PUSH_REGS(); \
		current->sp = SP; \
		switch_mm(); \
		SP = current->sp; \
		POP_REGS(); \
		POP_DP(); \
	} while (0)

#ifdef CONFIG_USER_PREEMPT
/**
 * irq_switch_task - switch task from interrupt.
 *
 * This function should be called after waking up process that should
 * preempt current process. It's also called in periodic timer interrupt
 * to implement round-robin scheduling.
 *
 * See TASK_SWITCH().
 */
#define irq_switch_task() \
	do { \
		if (irq_preempt_enabled() && (current->next != current)) \
			TASK_SWITCH(); \
	} while (0)
#else
#define irq_switch_task()	do { } while (0)
#endif

/**
 * current - current context
 */
extern struct context __kernel *current;

char context_init(struct context __kernel *context) __user_preempt;
char thread_create(struct context __kernel *context, unsigned char sp);
void thread_switch(void) __thread_switch_attr;
void show_context_state(struct context __kernel *context);

#else
#define irq_switch_task()	do { } while (0)
#endif

#endif
