/*
 * tmos - types.h 
 * Copyright (C) 2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Fundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef __ASM_TYPES_H
#define __ASM_TYPES_H

#include <ctype.h>
#include <stddef.h>
#include <limits.h>

#define DATA_NULL	((void __data *) 0)
#define XDATA_NULL	((void __xdata *) 0)

typedef unsigned char uint8_t;
typedef unsigned int uint16_t;
typedef unsigned long uint32_t;

#define UINT8_MAX	UCHAR_MAX
#define UINT16_MAX	UINT_MAX
#define UINT32_MAX	ULONG_MAX

typedef signed char int8_t;
typedef signed int int16_t;
typedef signed long int32_t;

#define INT8_MIN	SCHAR_MIN
#define INT16_MIN	INT_MIN
#define INT32_MIN	LONG_MIN
#define INT8_MAX	SCHAR_MAX
#define INT16_MAX	INT_MAX
#define INT32_MAX	LONG_MAX

typedef unsigned char u8;
typedef unsigned int u16;
typedef unsigned long u32;

typedef signed char s8;
typedef signed int s16;
typedef signed long s32;

#define bswap16(x)	(((x) >> 8) | ((x) << 8))

#define bswap32_const(x) \
	( (((x) >> 24) & 0x000000ff) \
	| (((x) >>  8) & 0x0000ff00) \
	| (((x) <<  8) & 0x00ff0000) \
	| (((x) << 24) & 0xff000000))

u32 __bswap32(u32 x) __naked;

#define bswap32(x)	(__builtin_constant_p(x) ? bswap32_const(x) \
		: __bswap32(x))

#ifdef BIG_ENDIAN
#define le16_to_cpu(x)		(bswap16(x))
#define le32_to_cpu(x)		(bswap32(x))

#define be16_to_cpu(x)		(x)
#define be32_to_cpu(x)		(x)

#define cpu_to_le16(x)		(bswap16(x))
#define cpu_to_le32(x)		(bswap32(x))

#define cpu_to_be16(x)		(x)
#define cpu_to_be32(x)		(x)
#else
#define le16_to_cpu(x)		(x)
#define le32_to_cpu(x)		(x)

#define be16_to_cpu(x)		(bswap16(x))
#define be32_to_cpu(x)		(bswap32(x))

#define cpu_to_le16(x)		(x)
#define cpu_to_le32(x)		(x)

#define cpu_to_be16(x)		(bswap16(x))
#define cpu_to_be32(x)		(bswap32(x))
#endif

typedef signed int ptrdiff_t;

#endif
