/*
 * tmos - console.h
 *
 * Copyright (C) 2007..2009  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef CONSOLE_H_INCLUDED
#define CONSOLE_H_INCLUDED

#include <tmos/config.h>

#if CONFIG_CONSOLE_SERIAL_LCDEMU
#include <tmos/ser_lcdemu.h>
#elif CONFIG_CONSOLE_SERIAL
#include <ser_console.h>
#elif CONFIG_CONSOLE_LCD
#include <tmos/lcd.h>
#else

#define CONSOLE_COLS		20
#define CONSOLE_ROWS		2

#define c_mkpos(row, col)	(0)

/* initialize console */
#define console_init()		do { } while (0)

/* put chararacter, _p version prints character at specified position */
#define c_putchar(c)		do { } while (0)
#define c_putchar_p(pos, c)	do { } while (0)

#define c_puts(s)		do { } while (0)
#define c_puts_p(pos, s)	do { } while (0)

#define CONSOLE_HAVE_PUTS
#define CONSOLE_HAVE_PUTS_P

#define c_setpos(pos)		do { } while (0)

#define c_flush()		do { } while (0)

#endif

#if defined(CONSOLE_FB_ONLY) && defined(CONSOLE_STREAM)
void c_putchar(char c) REENTRANT;

#ifndef CONSOLE_HAVE_PUTS
void c_puts(char *c) REENTRANT;
#endif
#endif


#ifndef CONSOLE_HAVE_PUTB
// void c_putb(char *buf, char len) REENTRANT;
#endif

#ifndef CONSOLE_HAVE_PUTB_P
void c_putb_p(char pos, char *buf, char len) REENTRANT;
#endif

char d_to_char(char x);

#ifdef CONSOLE_STREAM
#endif

#define CONSOLE_BLOCK

#ifdef CONSOLE_BLOCK

#ifndef CONSOLE_HAVE_PUTS_P
void c_puts_p(char pos, char *c) REENTRANT;
#endif

#ifndef CONSOLE_HAVE_PUTB_P
void c_putb_p(char pos, char *buf, char len) REENTRANT;
#endif

#ifndef CONSOLE_HAVE_PUT
#define c_put(buf)	c_putb_p(0, buf, CONSOLE_ROWS * CONSOLE_COLS)
#endif

/*
 * c_put_uint_p - put decimal unsigned integer
 * 	pos: end position
 * 	len: maximal length in characters
 * 	val: integer to print
 */
void c_put_uint_e(char pos, char len, unsigned int val) REENTRANT;

/*
 * c_put_uintz_p - put decimal integer with leading zeros
 * 	pos: end position
 * 	len: maximal length in characters
 * 	val: integer to print
 */
void c_put_uintz_e(char pos, char len, unsigned int val) REENTRANT;

/*
 * c_put_hchar_p - put decimal integer
 * 	pos: end position
 * 	val: integer to print
 */
void c_put_hchar_p(char pos, unsigned char val) REENTRANT;

/*
 * c_put_hcharz_p - put decimal integer with leading zeros
 * 	pos: end position
 * 	val: integer to print
 */
void c_put_hcharz_p(char pos, unsigned char val) REENTRANT;
#endif

#define LCD_COMPAT

#ifdef LCD_COMPAT

#define LCD_COLS	CONSOLE_COLS
#define LCD_ROWS	CONSOLE_ROWS

#define c_flush_after(x) \
	do { \
		x; \
		c_flush(); \
	} while (0)

#define lcd_putchar(pos, c)	c_flush_after(c_putchar_p(pos, c))
#define lcd_puts(pos, c)	c_flush_after(c_puts_p(pos, c))
#define lcd_putb(pos, buf, len)	c_flush_after(c_putb_p(pos, buf, len))
#define lcd_put(buf)		lcd_putb(0, buf, CONSOLE_ROWS * CONSOLE_COLS)

#define lcd_put_bcd(pos, v) c_flush_after(c_put_hchar_p(pos, v))
#define lcd_put_dec(pos, len, v) c_flush_after(c_put_uint_e(pos, len, v))
#define lcd_put_decz(pos, len, v) c_flush_after(c_put_uintz_e(pos, len, v))

#endif

#endif
